#  include "../config.h"

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <libintl.h>
#include <locale.h>
#define _(String) dgettext(PACKAGE, String)
#include "eggtrayicon.h"
#include "principals.h"
#include "common.h"

//check ticket every 30 seconds
#define CHECK_TICKET_INTERVAL (30*1000)
//warn(yellow icon) when less than 5 minutes till ticket expires
#define WARN_SECONDS (60*5)

static void view_popup_menu(GtkWidget * widget, GdkEventButton * event, gpointer userdata);

static GtkWidget *eventbox = NULL;
static gboolean mainwindow_destroyed = FALSE;
static EggTrayIcon *tray_icon = NULL;
static GtkTooltips *tray_icon_tips = NULL;
static GtkWidget *image = NULL;	
enum ticket_icon {
	NO_ICON = 0,
	GREEN_ICON,
	YELLOW_ICON,
	RED_ICON
};
//must match the enums above..
static const gchar *const icon_names[] = {
	DATADIR"/"PACKAGE"/missing_ticket.png",
	DATADIR"/"PACKAGE"/green_ticket.png",
	DATADIR"/"PACKAGE"/yellow_ticket.png",
	DATADIR"/"PACKAGE"/red_ticket.png"
};
static enum ticket_icon current_icon;

static void tray_icon_pressed(GtkWidget * widget, GdkEventButton * event, EggTrayIcon * icon)
{

	if (event->button == 3) {
		view_popup_menu(widget, event, icon);
	}

}
static void show_main_window(GtkMenuItem * menuitem, gpointer user_data)
{
	if (mainwindow_destroyed) {
		init_and_show_gui();
		mainwindow_destroyed = FALSE;
	}

}

static void view_popup_menu(GtkWidget * widget, GdkEventButton * event, gpointer userdata)
{
	GtkWidget *menu, *menuitem;

	menu = gtk_menu_new();
	menuitem = gtk_menu_item_new_with_mnemonic("_Show tickets");
	g_signal_connect(G_OBJECT(menuitem), "activate", G_CALLBACK(show_main_window), NULL);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);

	menuitem = gtk_menu_item_new_with_mnemonic("_About");
	gtk_widget_set_sensitive(menuitem,FALSE);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);

	menuitem = gtk_menu_item_new_with_mnemonic("_Quit");
	g_signal_connect(G_OBJECT(menuitem), "activate", G_CALLBACK(gtk_main_quit), NULL);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);

	gtk_widget_show_all(menu);
	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, (event != NULL) ? event->button : 0, gdk_event_get_time((GdkEvent *) event));
}

void set_mainwindow_destroyed(gboolean isdestroyed)
{
	mainwindow_destroyed = isdestroyed;
}

static gboolean check_ticket_time (gpointer data)
{
	GTimeVal now;
	guint32 endtime;
	gchar buf[256];
	gchar *p;
	enum ticket_icon theicon;
	g_get_current_time(&now);
	if(check_initial_ticket(&endtime) == NO_TICKET){
		theicon = NO_ICON;
		p = _("No tickets") ;
											  
	} else {
		if(now.tv_sec >= endtime){
			theicon = RED_ICON;
			p = _("Ticket expired") ;
		} else {
			guint32 diff = endtime - now.tv_sec;
			guint32 min,hour,day;
			if((diff) < WARN_SECONDS) {
				theicon = YELLOW_ICON;
			} else {
				theicon = GREEN_ICON;	
			}
			//step by step
			day =  diff/(24*3600);
			hour = diff - (day*24*3600);
			hour = hour/3600;
			min = day*24*3600+hour*3600;
			min = (diff - min)/60;
			
			g_snprintf(buf,sizeof(buf),_("Ticket expiring in %u days %u hours %u minutes"),day,hour,min);
			p = buf;
		}
	}

	if(current_icon != theicon){
		gtk_image_set_from_file(GTK_IMAGE(image),icon_names[theicon]);
		current_icon = theicon;
	}
	gtk_tooltips_set_tip(GTK_TOOLTIPS(tray_icon_tips),GTK_WIDGET(eventbox),p,p);
	return TRUE;
}

void setupTrayIcon()
{

	tray_icon = egg_tray_icon_new(PACKAGE);
	eventbox = gtk_event_box_new();
	image = gtk_image_new_from_file(icon_names[NO_ICON]);
	current_icon = NO_ICON;
	gtk_container_add(GTK_CONTAINER(eventbox), image);
	g_signal_connect(eventbox, "button_press_event", G_CALLBACK(tray_icon_pressed), tray_icon);
	gtk_container_add(GTK_CONTAINER(tray_icon), eventbox);
	tray_icon_tips = gtk_tooltips_new();
	gtk_widget_show_all(GTK_WIDGET(tray_icon));
	check_ticket_time(NULL);
	g_timeout_add(CHECK_TICKET_INTERVAL,check_ticket_time,NULL);
}
